@@ -2,6 +2,8 @@ package Quiki;
 
 use feature ':5.10';
 
+use Data::Dumper;
+
 use Quiki::Formatter;
 use Quiki::Meta;
 use Quiki::Users;
@@ -10,7 +12,6 @@ use Quiki::Pages;
 use CGI qw/:standard *div/;
 use CGI::Session;
 use HTML::Template::Pro;
-use Gravatar::URL;
 use File::MMagic;
 use File::Slurp 'slurp';
 
@@ -23,11 +24,11 @@ Quiki - A lightweight Wiki in Perl
 
 =head1 VERSION
 
-Version 0.01_2
+Version 0.02_1
 
 =cut
 
-our $VERSION = '0.01_2';
+our $VERSION = '0.02_1';
 
 
 =head1 SYNOPSIS
@@ -45,7 +46,7 @@ Creates a new Quiki object.
 
 =head2 run
 
-Runs de Quiki.
+Runs the Quiki.
 
 =cut
 
@@ -58,24 +59,23 @@ sub new {
     my %conf = (
                 name  => 'defaultName',
                 index => 'index', # index node
-                protocol => 'http', # support https?
                );
     $self = {%conf, %args};
 
     $self->{SCRIPT_NAME} = $ENV{SCRIPT_NAME};
     $self->{SERVER_NAME} = $ENV{SERVER_NAME};
 
-    $self->{DOCROOT} = sprintf("%s://%s%s",
-                               $self->{protocol}, $self->{SERVER_NAME}, $ENV{SCRIPT_NAME});
+    $self->{DOCROOT} = $ENV{SCRIPT_NAME};
     $self->{DOCROOT} =~ s!/[^/]+$!/!;
 
+    $self->{QUIKI_ID} = $self->{name} . '_' . slurp 'data/quiki_id';
     return bless $self, $class;
 }
 
 sub run {
     my $self = shift;
 
-    $self->{sid} = cookie("QuikiSID") || undef;
+    $self->{sid} = cookie($self->{QUIKI_ID}) || undef;
     $self->{session} = new CGI::Session(undef, $self->{sid}, undef);
 
     # XXX
@@ -88,10 +88,32 @@ sub run {
     $self->{meta} = Quiki::Meta::get($node);
     $self->{node} = $node;
 
+    if ($action eq 'update_perms') {
+        my $username = param('edit_user');
+        if (param("admin_action") eq "Delete") {
+            if ($username eq "admin") {
+                $self->{session}->param('msg', "Admin account can not be deleted.");
+            } else {
+                Quiki::Users->delete($username);
+                $self->{session}->param('msg', "User '$username' deleted.");
+            }
+        }
+        if (param("admin_action") eq "Save") {
+            my $perm = param("perms");
+            if ($username eq "admin") {
+                $self->{session}->param('msg', "Admin account righs can not be changed.");
+            } else {
+                Quiki::Users->update($username, perm_group => $perm);
+                $self->{session}->param('msg', "Permission rights changed for user '$username'.");
+            }
+        }
+        $action = 'admin_page';
+    }
 
     if ($action eq 'save_profile' && param('submit') =~ /^Save/) {
         if (param("new_password1") && (param("new_password1") ne param("new_password2"))) {
             $self->{session}->param('msg', "Passwords do not match. Try again!");
+            $action = 'profile_page';
         }
         else {
             my %data;
@@ -136,6 +158,7 @@ sub run {
             if (Quiki::Users->exists($username)) {
                 $self->{session}->param('msg',
                                         "User name already in use. Please try again!");
+                $action = 'register_page';
             }
             else {
                 Quiki::Users->create($self, $username, $email);
@@ -146,6 +169,7 @@ sub run {
         else {
             $self->{session}->param('msg',
                                     "Sign up failed! Perhaps you forgot to fill in the form?");
+            $action = 'register_page';
         }
     }
 
@@ -174,7 +198,6 @@ sub run {
     # XXX
     ($action eq 'create') and (-f "data/content/$node") and ($action = '');
     if( ($action eq 'create') or !-f "data/content/$node") {
-        #Quiki::Pages -> save($node, "Edit me!");
         Quiki::Pages->check_in($self, $node, "Edit me!");
 	$self->{session}->param('msg',"New node \"$node\" created.");
     }
@@ -188,7 +211,6 @@ sub run {
     if ($action eq 'save' && param("submit") eq "Save") {
         if (Quiki::Pages->locked($node, $self->{sid})) {
             my $text = param('text') // '';
-            #Quiki::Pages->save($node, $text);
             Quiki::Pages->check_in($self, $node, $text);
             Quiki::Pages->unlock($node);
             $self->{session}->param('msg',"Content for \"$node\" updated.");
@@ -200,20 +222,27 @@ sub run {
     # XXX
     my $content;
     $self->{rev} = param('rev') || $self->{meta}{rev};
+	# sanity check revision number
+	if (!($self->{rev} =~ m/\d+/) || $self->{rev}<0 || $self->{rev}>$self->{meta}{rev}) {
+    	$self->{rev} = $self->{meta}{rev};
+		$self->{session}->param('msg','Revision requested not found.');
+	}
     if ($action eq 'rollback') {
         $content = Quiki::Pages->check_out($self,$node,$self->{rev});
         Quiki::Pages->check_in($self, $node, $content);
+        $self->{rev} = $self->{meta}{rev};
     }
     else {
     	$content = Quiki::Pages->check_out($self,$node,$self->{rev});
     }
 
 
-    my $cookie = cookie('QuikiSID' => $self->{session}->id);
+    my $cookie = cookie($self->{QUIKI_ID} => $self->{session}->id);
     print header(-charset=>'UTF-8',-cookie=>$cookie);
 
     my @trace;
     $self->{session}->param('trace') and @trace = @{$self->{session}->param('trace')};
+	!@trace and push @trace, $node;
     if ($trace[-1] ne $node) {
         push @trace, $node;
         @trace > 5 and shift @trace;
@@ -233,7 +262,9 @@ sub run {
 
     my $username = ($self->{session}->param('authenticated')?
                     $self->{session}->param('username'):"guest");
+    ## XXX - Later, join these functions to query database only once
     my $email    = Quiki::Users->email($username);
+    my $urole    = Quiki::Users->role($username);
     my $theme    = $self->{theme} || 'default';
 
     my $template = HTML::Template::Pro->new(filename => "themes/$theme/wrapper.tmpl",
@@ -249,12 +280,13 @@ sub run {
                      REV         => $self->{rev},
                      BREADCUMBS  => $breadcumbs,
                      DOCROOT     => $self->{DOCROOT},
+                     USER_ROLE   => $urole,
                      PREVIEW     => $preview,
                     );
 
     if ($action eq 'profile_page') {
         $template->param(EMAIL       => $email,
-                         GRAVATAR    => gravatar_url(email => $email));
+                         GRAVATAR    => Quiki::Users->gravatar($username));
     }
 
     if ($action eq 'edit' && 
@@ -262,26 +294,38 @@ sub run {
         if ($preview) {
             my $text = param('text') // '';
             $template->param(CONTENT=>Quiki::Formatter::format($self, $text));
+        	$template->param(TEXT=>$text);
         }
         else {
             Quiki::Pages->lock($node, $self->{sid});
+        	$template->param(TEXT=>$content);
         }
 
-        $template->param(TEXT=>$content);
-
         if (-d "data/attach/$node") {
             my @attachs;
             opendir DIR, "data/attach/$node";
             my $mm = new File::MMagic;
             my %desc;
-            for my $f (sort { lc($a) <=> lc($b)  } readdir(DIR)) {
+            for my $f (sort { lc($a) cmp lc($b)  } readdir(DIR)) {
                 next if $f =~ /^\.\.?$/;
                 if ($f =~ m!_desc_(.*)!) {
                     $desc{$1} = slurp "data/attach/$node/$f";
                 }
                 else {
                     my $mime = $mm->checktype_filename("data/attach/$node/$f");
-                    push @attachs, { ID => $f, MIME => $mime };
+                    my $mimeimg;
+                    given ($mime) {
+                        when (/image/) {
+                            $mimeimg = "images.png"
+                        }
+                        when (/pdf/) {
+                            $mimeimg = "doc_pdf.png"
+                        }
+                        default {
+                            $mimeimg = "page.png"
+                        }
+                    }
+                    push @attachs, { ID => $f, MIME => $mime, MIMEIMG => $mimeimg};
                 }
             }
             for (@attachs) {
@@ -290,20 +334,51 @@ sub run {
             $template->param(ATTACHS => \@attachs);
         }
     }
-    elsif ($action eq 'index') {
-        opendir(DIR,'data/content/');
-        my @pages;
-        for my $f (sort { lc($a) <=> lc($b) } readdir(DIR)) {
-            unless ($f=~/^\./) {
-                push @pages, { link => a({-href=>"$self->{SCRIPT_NAME}?node=$f"}, $f)};
+    elsif ($action eq 'history') {
+
+        my @revs;
+        for (my $i=$self->{meta}{rev} ; $i>0 ; $i--) {
+            my $entry = { VERSION => $i };
+            if ($i != $self->{meta}{rev}) {
+                $entry->{AUTHOR} = $self->{meta}{revs}{$i}{last_update_by};
+                $entry->{DATE} =  $self->{meta}{revs}{$i}{last_updated_in};
+                $entry->{GRAVATAR} = Quiki::Users->gravatar($self->{meta}{revs}{$i}{last_update_by});
             }
+            else {
+                $entry->{AUTHOR} = $self->{meta}{last_update_by};
+                $entry->{DATE} =  $self->{meta}{last_updated_in};
+                $entry->{GRAVATAR} = Quiki::Users->gravatar($self->{meta}{last_update_by});
+            }
+            push @revs, $entry;
         }
-        closedir(DIR);
+        $template->param(REVISIONS => \@revs);
+    }
+    elsif ($action eq 'admin_page') {
+        my $users = Quiki::Users->list;
+        $template->param(USERS => $users);
+    }
+    elsif ($action eq 'index') {
+        opendir DIR, 'data/content/';
+        my @pages;
+        for my $f (sort { lc($a) cmp lc($b) } readdir(DIR)) {
+            unless ($f=~/^\./) {
+                my $meta = Quiki::Meta::get($f);
+                push @pages, 
+                  { URL => "$self->{SCRIPT_NAME}?node=$f",
+                    NAME => $f,
+                    AUTHOR => $meta->{last_update_by},
+                    DATE => $meta->{last_updated_in},
+                    GRAVATAR => Quiki::Users->gravatar($meta->{last_update_by}),
+                  }
+              }
+        }
+        closedir DIR;
         $template->param(PAGES=>\@pages);
     }
     elsif ($action eq 'diff') {
-        my $target = param('target') || 0;
-        $template->param(CONTENT=>Quiki::Pages->calc_diff($self,$node,$self->{rev},$target));
+        my $source = param('source') || 1;
+        my $target = param('target') || 1;
+        $template->param(CONTENT=>Quiki::Pages->calc_diff($self,$node,$source,$target));
     }
     else {
         $template->param(CONTENT=>Quiki::Formatter::format($self, $content));
@@ -318,28 +393,19 @@ sub run {
     }
 
     unless ($action eq 'edit') {
-        my $L_META = sprintf("Last edited by %s, in %s",
-                             $self->{meta}{last_update_by}  || "",
-                             $self->{meta}{last_updated_in} || "");
-        my $R_META = sprintf("Revision: %s | Older: ",
-                             $self->{meta}{rev} || "");
-
-        if ($self->{meta}{rev} > 1) {
-            for (my $i=$self->{meta}{rev} ; $i>0 ; $i--) {
-                $R_META .= a({-href=>"$self->{SCRIPT_NAME}?node=$node&rev=$i"}, $i).' ';
-            }
-            $R_META .= start_form(-method => 'post',
-                                  -action => $self->{SCRIPT_NAME},
-                                  -style  => 'display: inline;');
-            $R_META .= hidden(-name => 'node', -value => $node, -override => 1);
-            $R_META .= hidden(-name => 'action', -value => 'diff', -override => 1);
-            $R_META .= submit(-name => 'submit', -value => 'Calc diff with: ', -override => 1);
-            $R_META .= "<select name='target'>";
-            for (my $i=$self->{meta}{rev}-1 ; $i>0 ; $i--) {
-                $R_META .= "<option value='$i'>revision $i</option>";
-            }
+        my $L_META;
+        if ($self->{meta}{last_update_by}) {
+            my $url = Quiki::Users->gravatar($self->{meta}{last_update_by});
+            $L_META = img({-src => $url, -width => '24', -style => 'vertical-align: middle'});
+            $L_META .= sprintf("&nbsp;Last edited by %s, in %s",
+                               $self->{meta}{last_update_by},
+                               $self->{meta}{last_updated_in} || "");
         }
-        $R_META .= "</select></form>";
+        else {
+            $L_META = "";
+        }
+        my $R_META = sprintf("Revision: %s", $self->{meta}{rev} || "");
+
         $template->param(L_META=>$L_META);
         $template->param(R_META=>$R_META);
     }
@@ -351,7 +417,6 @@ sub run {
 
     # save meta data
     Quiki::Meta::set($node, $self->{meta});
-
     $template->output(print_to => \*STDOUT);
 }
 
