@@ -121,10 +121,11 @@ sub _format_chunk {
 
     if ($chunk =~ /^(\^|\|)/) {
         $chunk = _format_table($Quiki, $chunk);
+        $chunk = _unbackslash($chunk);
     }
     elsif ($chunk =~ /^\s{2}[*-]/) {
         $chunk = _format_list($Quiki, $chunk);
-
+        $chunk = _unbackslash($chunk);
     }
     elsif ($chunk =~ /^\s{3}/) {
         $chunk = _format_verbatim($chunk);
@@ -220,9 +221,14 @@ sub _inlines {
        qr/'' ((?:\\'|[^']|'[^'])+) ''/x => sub { tt(_inlines($Quiki, $1)) },
 
        ## {{wiki: foo | desc }}
-       qr/\{\{wiki:([^}|]+)\|([^}]+)\}\}/        => sub { _inline_doc($Quiki, $1,$2) },
+       qr/\{\{(\s*)wiki:([^}|]+)\|([^}]+?)(\s*)\}\}/        => sub {
+           my $align = (length($1) && length($4))?"center":(length($1)?"right":"left");
+           _inline_doc($Quiki, $2,$3, $align)
+       },
        ## {{wiki: foo  }}
-       qr/\{\{wiki:([^}]+)\}\}/                  => sub { _inline_doc($Quiki, $1,$1) },
+       qr/\{\{(\s*)wiki:([^}]+?)(\s*)\}\}/                  => sub {
+           my $align = (length($1) && length($3))?"center":(length($1)?"right":"left");
+           _inline_doc($Quiki, $2,$2, $align) },
 
        ## {{ foo | desc  }}
        qr/\{\{([^}|]+)\|([^}]+)\}\}/        => sub { img({alt => $2, src => $1}) },
@@ -244,12 +250,23 @@ sub _inlines {
 }
 
 sub _inline_doc {
-    my ($quiki, $id, $desc) = @_;
+    my ($quiki, $id, $desc, $align) = @_;
     my $node = $quiki->{node};
     my $mm = new File::MMagic;
     my $mime = $mm->checktype_filename("data/attach/$node/$id");
     if ($mime =~ /^image/) {
-        img({-alt=>$desc, -src=>"data/attach/$node/$id"})
+        given($align) {
+            when ("right") {
+                return img({-alt=>$desc, -src=>"data/attach/$node/$id", -style=>"float: right"})
+            }
+            when ("center") {
+                return div({-style=>"text-align: center"}, 
+                           img({-alt=>$desc, -src=>"data/attach/$node/$id"}))
+            }
+            default {
+                return img({-alt=>$desc, -src=>"data/attach/$node/$id"})
+            }
+        }
     }
     else {
         a({-href=>"data/attach/$node/$id", -target=>"_new"},
